@@ -1,9 +1,15 @@
+/* Define if symbols are prefixed with underscore */
+#undef SYMBOL_UNDERSCORE
+
 /* Tag to identify path to libgii.conf */
 #define GIIPATHTAG	FIXME_DEFINE_THIS!
 
-/* Tag plus full path to libgii.conf */
+/* Name of LibGII master config file */
 #define GIICONFFILE	FIXME_DEFINE_THIS!
 
+/* Tag plus path to config dir */
+#define GIICONFDIR	FIXME_DEFINE_THIS!
+
 /* The length of the tag */
 #define GIITAGLEN	FIXME_DEFINE_THIS!
 
