@@ -1,4 +1,4 @@
-/* $Id: packetparse.c,v 1.8 1998/11/19 00:47:19 marcus Exp $
+/* $Id: packetparse.c,v 1.15 1999/05/16 15:48:41 marcus Exp $
 ******************************************************************************
 
    Mouse packet parsing
@@ -39,13 +39,13 @@ static void mouse_send_movement(gii_input *inp, sint32 dx, sint32 dy,
 {
 	gii_event ev;
 
-	if (dx || dy || dz || wheel) {
-		_giiEventBlank(&ev);
+	if ((inp->curreventmask & emPtrRelative) &&
+	    (dx || dy || dz || wheel)) {
+		_giiEventBlank(&ev, sizeof(gii_pmove_event));
 
 		ev.pmove.size   = sizeof(gii_pmove_event);
 		ev.pmove.type   = evPtrRelative;
 		ev.pmove.origin = inp->origin;
-		ev.pmove.target = GII_EV_TARGET_ALL;
 
 		ev.pmove.x = dx;
 		ev.pmove.y = dy;
@@ -67,19 +67,26 @@ static void mouse_send_buttons(gii_input *inp, uint32 buttons, uint32 last)
 
 	for (nr = 1, mask = 1; mask != 0; nr++, mask <<= 1) {
 		if (changed & mask) {
-			_giiEventBlank(&ev);
+			_giiEventBlank(&ev, sizeof(gii_pbutton_event));
 
 			if (buttons & mask) {
+				if (! (inp->curreventmask
+				       & emPtrButtonPress)) {
+					continue;
+				}
 				ev.pbutton.type = evPtrButtonPress;
 				MOUSE_PRIV(inp)->sent |= emPtrButtonPress;
 			} else {
+				if (! (inp->curreventmask
+				       & emPtrButtonRelease)) {
+					continue;
+				}
 				ev.pbutton.type = evPtrButtonRelease;
 				MOUSE_PRIV(inp)->sent |= emPtrButtonRelease;
 			}			
 
 			ev.pbutton.size = sizeof(gii_pbutton_event);
 			ev.pmove.origin = inp->origin;
-			ev.pmove.target = GII_EV_TARGET_ALL;
 			ev.pbutton.button = nr;
 			
 			_giiEvQueueAdd(inp, &ev);
@@ -112,7 +119,7 @@ static int parse_ms(gii_input *inp, uint8 *buf, int len)
 
 	/* check header */
 	if (((buf[0] & 0x40) != 0x40) || ((buf[1] & 0x40) != 0x00)) {
-		DPRINT_EVENTS("Invalid microsoft packet\n");
+		GIIDPRINT_EVENTS("Invalid microsoft packet\n");
 		return 1;
 	}
 
@@ -128,7 +135,7 @@ static int parse_ms(gii_input *inp, uint8 *buf, int len)
 		mpriv->button_state = buttons;
 	}
 
-	DPRINT_EVENTS("Got microsoft packet\n");
+	GIIDPRINT_EVENTS("Got microsoft packet\n");
 
 	return 3;
 } 
@@ -149,7 +156,7 @@ static int parse_ms3(gii_input *inp, uint8 *buf, int len)
 		int dx, dy;
 		/* check header */
 		if (((buf[0] & 0x40) != 0x40) || ((buf[1] & 0x40) != 0x00)) {
-			DPRINT_EVENTS("Invalid IntelliMouse packet\n");
+			GIIDPRINT_EVENTS("Invalid IntelliMouse packet\n");
 			return 1;
 		}
 		
@@ -165,7 +172,7 @@ static int parse_ms3(gii_input *inp, uint8 *buf, int len)
 			mouse_send_buttons(inp, buttons, mpriv->button_state);
 			mpriv->button_state = buttons;
 		}
-		DPRINT_EVENTS("Got IntelliMouse base packet\n");
+		GIIDPRINT_EVENTS("Got IntelliMouse base packet\n");
 	}
 
 	if (len < 4) {
@@ -176,7 +183,7 @@ static int parse_ms3(gii_input *inp, uint8 *buf, int len)
 
 	if ((buf[3] & 0x40) == 0x40) {
 		/* 4th byte must be a header byte */
-		DPRINT_EVENTS("Got 3-byte IntelliMouse packet\n");
+		GIIDPRINT_EVENTS("Got 3-byte IntelliMouse packet\n");
 		return 3;
 	}
 
@@ -192,7 +199,7 @@ static int parse_ms3(gii_input *inp, uint8 *buf, int len)
 		mpriv->button_state = buttons;
 	}
 
-	DPRINT_EVENTS("Got 4-byte IntelliMouse packet\n");
+	GIIDPRINT_EVENTS("Got 4-byte IntelliMouse packet\n");
 	
 	return 4;
 } 
@@ -214,7 +221,7 @@ static int parse_msc(gii_input *inp, uint8 *buf, int len)
 
 	/* check header */
 	if ((buf[0] & 0xf8) != 0x80) {
-		DPRINT_EVENTS("Invalid mousesys packet\n");
+		GIIDPRINT_EVENTS("Invalid mousesys packet\n");
 		return 1;
 	}
 
@@ -229,7 +236,7 @@ static int parse_msc(gii_input *inp, uint8 *buf, int len)
 		mpriv->button_state = buttons;
 	}
 
-	DPRINT_EVENTS("Got mousesys packet\n");
+	GIIDPRINT_EVENTS("Got mousesys packet\n");
 
 	return 5;
 }
@@ -251,7 +258,7 @@ static int parse_logi(gii_input *inp, uint8 *buf, int len)
 
 	/* check header */
 	if (((buf[0] & 0xe0) != 0x80) || ((buf[1] & 0x80) != 0x00)) {
-		DPRINT_EVENTS("Invalid logitech packet\n");
+		GIIDPRINT_EVENTS("Invalid logitech packet\n");
 		return 1;
 	}
 
@@ -266,7 +273,7 @@ static int parse_logi(gii_input *inp, uint8 *buf, int len)
 		mpriv->button_state = buttons;
 	}
 
-	DPRINT_EVENTS("Got logitech packet\n");
+	GIIDPRINT_EVENTS("Got logitech packet\n");
 
 	return 3;
 }
@@ -288,7 +295,7 @@ static int parse_sun(gii_input *inp, uint8 *buf, int len)
 
 	/* check header */
 	if ((buf[0] & 0xf8) != 0x80) {
-		DPRINT_EVENTS("Invalid sun packet\n");
+		GIIDPRINT_EVENTS("Invalid sun packet\n");
 		return 1;
 	}
 
@@ -303,7 +310,7 @@ static int parse_sun(gii_input *inp, uint8 *buf, int len)
 		mpriv->button_state = buttons;
 	}
 
-	DPRINT_EVENTS("Got sun packet\n");
+	GIIDPRINT_EVENTS("Got sun packet\n");
 
 	return 3;
 }
@@ -342,7 +349,7 @@ static int parse_mman(gii_input *inp, uint8 *buf, int len)
 
 	/* check header */
 	if (((buf[0] & 0x40) != 0x40) || ((buf[1] & 0x40) != 0x00)) {
-		DPRINT_EVENTS("Invalid mouseman packet\n");
+		GIIDPRINT_EVENTS("Invalid mouseman packet\n");
 		return 1;
 	}
 
@@ -361,7 +368,7 @@ static int parse_mman(gii_input *inp, uint8 *buf, int len)
 		mpriv->button_state = buttons;
 		mpriv->parse_state  = 1;
 
-		DPRINT_EVENTS("Got mouseman base packet\n");
+		GIIDPRINT_EVENTS("Got mouseman base packet\n");
 	}
 
 	/* now look for extension byte */
@@ -384,7 +391,7 @@ static int parse_mman(gii_input *inp, uint8 *buf, int len)
 
 	mpriv->button_state = buttons;
 
-	DPRINT_EVENTS("Got mouseman extension packet\n");
+	GIIDPRINT_EVENTS("Got mouseman extension packet\n");
 
 	return 4;
 }
@@ -403,7 +410,7 @@ static int parse_ps2(gii_input *inp, uint8 *buf, int len)
 
 	/* Check header byte. */
 	if ((buf[0] & 0xc0) != 0) {
-		DPRINT_EVENTS("Invalid PS/2 packet\n");
+		GIIDPRINT_EVENTS("Invalid PS/2 packet\n");
 		return 1;
 	}
 
@@ -418,7 +425,7 @@ static int parse_ps2(gii_input *inp, uint8 *buf, int len)
 		mpriv->button_state = buttons;
 	}
 
-	DPRINT_EVENTS("Got PS/2 packet\n");
+	GIIDPRINT_EVENTS("Got PS/2 packet\n");
 
 	return 3;
 }
@@ -444,7 +451,7 @@ static int parse_mmanps2(gii_input *inp, uint8 *buf, int len)
 			: (buf[2] & 0x0f) - 16;
 	} else {
 		if ((buf[0] & 0xc0) != 0) {
-			DPRINT_EVENTS("Invalid MouseMan+ PS/2 packet\n");
+			GIIDPRINT_EVENTS("Invalid MouseMan+ PS/2 packet\n");
 			return 1;
 		}
 		buttons = (buf[0] & 0x07)
@@ -459,7 +466,7 @@ static int parse_mmanps2(gii_input *inp, uint8 *buf, int len)
 		mpriv->button_state = buttons;
 	}
 
-	DPRINT_EVENTS("Got MouseMan+ PS/2 packet\n");
+	GIIDPRINT_EVENTS("Got MouseMan+ PS/2 packet\n");
 
 	return 3;
 }
@@ -477,7 +484,7 @@ static int parse_imps2(gii_input *inp, uint8 *buf, int len)
 	mouse_priv *mpriv = MOUSE_PRIV(inp);
 
 	if ((buf[0] & 0xc0) != 0) {
-		DPRINT_EVENTS("Invalid IntelliMouse PS/2 packet\n");
+		GIIDPRINT_EVENTS("Invalid IntelliMouse PS/2 packet\n");
 		return 1;
 	}
 
@@ -492,7 +499,7 @@ static int parse_imps2(gii_input *inp, uint8 *buf, int len)
 		mpriv->button_state = buttons;
 	}
 
-	DPRINT_EVENTS("Got IntelliMouse PS/2 packet\n");
+	GIIDPRINT_EVENTS("Got IntelliMouse PS/2 packet\n");
 
 	return 3;
 }
@@ -534,7 +541,7 @@ static uint8 initdata_mmanps2[] =
   PS2_SCALE11, PS2_SETRES, 3, PS2_SETRES, 1, PS2_SETRES, 2, PS2_SETRES, 3,
 /* Make sure the mouse is enabled and in a sane state */
   PS2_DEFAULT, PS2_SCALE11, PS2_ENABLE,
-/* The MousMan+ is really slow by default, so we increase the resolution */
+/* The MouseMan+ is really slow by default, so we increase the resolution */
   PS2_SETRES, 3 };
 
 /* Mouse parsers
@@ -542,34 +549,34 @@ static uint8 initdata_mmanps2[] =
 */
 parser_type _gii_mouse_parsers[] = {
 	{ { "null", NULL }, parse_null, 0,
-	  NULL, 0 },
+	  NULL, 0, 0 },
 	{ { "ms", "Microsoft", NULL }, parse_ms, 3,
-	  NULL, 0 },
+	  NULL, 0, 0 },
 	{ { "msc", "MouseSystems", NULL }, parse_msc, 5,
-	  NULL, 0 },
+	  NULL, 0, 0 },
 	/* Logitech and MMSeries use the same protocol */
 	{ { "logi", "Logitech", "mm", "MMSeries", NULL }, parse_logi, 3,
-	  NULL, 0 },
+	  NULL, 0, 0 },
 	/* Sun and busmice use the same protocol */
 	{ { "sun", "bm", "BusMouse", NULL }, parse_sun, 3,
-	  NULL, 0 },
+	  NULL, 0, 0 },
 	{ { "mman", "MouseMan", NULL }, parse_mman, 3,
-	  NULL, 0 },
+	  NULL, 0, 0 },
 	/* Serial IntelliMouse or MouseMan+ */
 	{ { "ms3", "IntelliMouse", "mman+", NULL }, parse_ms3, 3,
-	  NULL, 0 },
+	  NULL, 0, 0 },
 
 	{ { "ps2", "PS/2", NULL }, parse_ps2, 3,
-	  initdata_ps2, sizeof(initdata_ps2) },
+	  initdata_ps2, sizeof(initdata_ps2), 0 },
 	/* PS/2 MouseMan+ */
 	{ { "mmanps2", "MouseManPlusPS/2", NULL }, parse_mmanps2, 3,
-	  initdata_mmanps2, sizeof(initdata_mmanps2) },
+	  initdata_mmanps2, sizeof(initdata_mmanps2), 1 },
 	/* PS/2 IntelliMouse */
 	{ { "imps2", "IMPS/2", NULL }, parse_imps2, 4,
-	  initdata_imps2 , sizeof(initdata_imps2) },
+	  initdata_imps2 , sizeof(initdata_imps2), 1 },
 
 	/* Terminator */
-	{ { NULL }, NULL, -1, NULL, 0 }
+	{ { NULL }, NULL, -1, NULL, 0, 0 }
 };
 
 
@@ -598,15 +605,38 @@ static int do_parse_packet(gii_input *inp)
 	used = mpriv->parser(inp, mpriv->packet_buf, 
 			     mpriv->packet_len);
 
-	DPRINT_EVENTS("packet used %d bytes\n", used);
+	GIIDPRINT_EVENTS("packet used %d bytes\n", used);
 
 	return used;
 }
 
-static gii_event_mask GII_mouse_handle_data(gii_input *inp)
+
+gii_event_mask GII_mouse_poll(gii_input *inp, void *arg)
 {
 	mouse_priv *mpriv = MOUSE_PRIV(inp);
 	int read_len;
+
+	GIIDPRINT_EVENTS("GII_mouse_poll(%p, %p) called\n", inp, arg);
+	
+	if (mpriv->eof) {
+		/* End-of-file, don't do any polling */
+		return 0;
+	}
+
+	if (arg == NULL) {
+		fd_set fds = inp->fdset;
+		struct timeval tv = { 0, 0 };
+		if (select(inp->maxfd, &fds, NULL, NULL, &tv) <= 0) {
+			return 0;
+		}
+	} else {
+		if (! FD_ISSET(mpriv->fd, ((fd_set*)arg))) {
+			/* Nothing to read on our fd */
+			GIIDPRINT_EVENTS("GII_mouse_poll: dummypoll\n");
+			return 0;
+		}
+	}
+		
 	mpriv->sent = 0;
 
 	/* read the mouse data */
@@ -616,12 +646,14 @@ static gii_event_mask GII_mouse_handle_data(gii_input *inp)
 	read_len = read(mpriv->fd, mpriv->packet_buf + mpriv->packet_len, 
 			read_len);
 
-	if (read_len == 0) {
-		/* End-of-file occured */
-		mpriv->eof = 1;
-		DPRINT_EVENTS("Mouse: EOF occured on fd: %d\n", mpriv->fd);
-		return 0;
-	} else if (read_len < 0) {
+	if (read_len <= 0) {
+		if (read_len == 0) {
+			/* End-of-file occured */
+			mpriv->eof = 1;
+			GIIDPRINT_EVENTS("Mouse: EOF occured on fd: %d\n",
+					 mpriv->fd);
+			return 0;
+		}
 		/* Error, we try again next time */
 		perror("Mouse: Error reading mouse");
 		return 0;
@@ -649,40 +681,7 @@ static gii_event_mask GII_mouse_handle_data(gii_input *inp)
 		}
 	}
 
-	DPRINT_EVENTS("Mouse: mouse_handle_data() done.\n");
+	GIIDPRINT_EVENTS("GII_mouse_poll: done\n");
 
 	return mpriv->sent;
 }
-
-
-gii_event_mask GII_mouse_poll(gii_input *inp)
-{
-	mouse_priv *mpriv = MOUSE_PRIV(inp);
-	gii_event_mask result = 0;
-
-	if (mpriv->eof) {
-		/* End-of-file, don't do any polling */
-		return 0;
-	}
-		
-	DPRINT_EVENTS("Mouse: poll(%p)\n", inp);
-	
-	for (;;) {
-		fd_set readset;
-		struct timeval t = {0,0};
-		int rc;
-
-		FD_ZERO(&readset);
-		FD_SET(mpriv->fd, &readset);
-
-		rc = select(inp->maxfd, &readset, NULL, NULL, &t);
-
-		if (rc <= 0) break;
-
-		result |= GII_mouse_handle_data(inp);
-
-		if (mpriv->eof) break;
-	}
-
-	return result;
-}
